@@ -1,9 +1,10 @@
-#![cfg_attr(all(test, feature = "i128"), feature(i128, i128_type))]
+#![cfg_attr(all(test, feature = "nightly"), feature(i128, i128_type))]
+#![cfg_attr(all(feature = "nightly"), feature(const_fn))]
 
 //! A library for defining enums that can be used in compact bit sets.
 //!
 //! It supports enums up to 64 variants on stable Rust, and up to 128 variants on nightly Rust with
-//! the `i128` feature enabled.
+//! the `nightly` feature enabled.
 //!
 //! # Defining enums for use with EnumSet
 //!
@@ -14,7 +15,6 @@
 //! # use enumset::*;
 //! enum_set_type! {
 //!     /// Documentation for the enum
-//!     #[derive(Debug)]
 //!     pub enum Enum {
 //!         A, B, C, D, E, F, G,
 //!         #[doc(hidden)] __NonExhaustive,
@@ -32,10 +32,9 @@
 //! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
 //! # enum_set_type! {
-//! #     #[derive(Debug)]
-//! #     pub enum Enum {
-//! #         A, B, C, D, E, F, G,
-//! #     }
+//! #      pub enum Enum {
+//! #        A, B, C, D, E, F, G,
+//! #    }
 //! # }
 //! # fn main() {
 //! let new_set = Enum::A | Enum::C | Enum::G;
@@ -49,10 +48,7 @@
 //! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
 //! # enum_set_type! {
-//! #    #[derive(Debug)]
-//! #    pub enum Enum {
-//! #        A, B, C, D, E, F, G,
-//! #    }
+//! #     enum Enum { A, B, C }
 //! # }
 //! # fn main() {
 //! const CONST_SET: EnumSet<Enum> = enum_set!(Enum, Enum::A | Enum::B);
@@ -66,8 +62,7 @@
 //! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
 //! # enum_set_type! {
-//! #    #[derive(Debug)]
-//! #    pub enum Enum {
+//! #      pub enum Enum {
 //! #        A, B, C, D, E, F, G,
 //! #    }
 //! # }
@@ -86,10 +81,9 @@
 //! [`enum_set!`]: ./macro.enum_set.html
 //! [`enum_set_type!`]: ./macro.enum_set_type.html
 
-use std::cmp::*;
 use std::fmt;
 use std::fmt::{Debug, Formatter};
-use std::hash::*;
+use std::hash::Hash;
 use std::ops::*;
 
 #[doc(hidden)]
@@ -109,7 +103,7 @@ pub trait EnumSetType : Copy {
 
 /// An efficient set type for enums created with the [`enum_set_type!`](./macro.enum_set_type.html)
 /// macro.
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash)]
 pub struct EnumSet<T : EnumSetType>(#[doc(hidden)] pub T::Repr);
 impl <T : EnumSetType> EnumSet<T> {
     fn mask(bit: u8) -> T::Repr {
@@ -121,10 +115,17 @@ impl <T : EnumSetType> EnumSet<T> {
     }
 
     /// Returns an empty set.
+    #[cfg(not(feature = "nightly"))]
     pub fn new() -> Self {
         EnumSet(T::ZERO)
     }
 
+    /// Returns an empty set.
+    #[cfg(feature = "nightly")]
+    pub const fn new() -> Self {
+        EnumSet(T::ZERO)
+    }
+
     /// Returns the number of values in this set.
     pub fn len(&self) -> usize {
         T::count_ones(self.0)
@@ -237,28 +238,6 @@ impl <T : EnumSetType> BitOr<T> for EnumSet<T> {
     }
 }
 
-impl <T : EnumSetType> PartialOrd for EnumSet<T> {
-    fn partial_cmp(&self, other: &EnumSet<T>) -> Option<Ordering> {
-        self.0.partial_cmp(&other.0)
-    }
-}
-impl <T : EnumSetType> Ord for EnumSet<T> {
-    fn cmp(&self, other: &Self) -> Ordering {
-        self.0.cmp(&other.0)
-    }
-}
-impl <T : EnumSetType> PartialEq for EnumSet<T> {
-    fn eq(&self, other: &EnumSet<T>) -> bool {
-        self.0.eq(&other.0)
-    }
-}
-impl <T : EnumSetType> Eq for EnumSet<T> { }
-impl <T : EnumSetType> Hash for EnumSet<T> {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.0.hash(state)
-    }
-}
-
 impl <T : EnumSetType + Debug> Debug for EnumSet<T> {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         let mut is_first = true;
@@ -297,7 +276,7 @@ impl <T : EnumSetType> Iterator for EnumSetIter<T> {
 
 #[macro_export]
 #[doc(hidden)]
-#[cfg(feature = "i128")]
+#[cfg(feature = "nightly")]
 macro_rules! enum_set_type_internal_count_variants {
     ($next:ident ($($args:tt)*)
         $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
@@ -362,7 +341,7 @@ macro_rules! enum_set_type_internal_count_variants {
 
 #[macro_export]
 #[doc(hidden)]
-#[cfg(not(feature = "i128"))]
+#[cfg(not(feature = "nightly"))]
 macro_rules! enum_set_type_internal_count_variants {
     ($next:ident ($($args:tt)*)
         $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
@@ -418,7 +397,7 @@ macro_rules! enum_set_type_internal {
         $($(#[$attr:meta])* $variant:ident,)*
     }) => {
         $(#[$enum_attr])* #[repr(u8)]
-        #[derive(Copy, Clone)]
+        #[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash, Debug)]
         $($vis)* enum $enum_name {
             $($(#[$attr])* $variant,)*
         }
@@ -452,9 +431,6 @@ macro_rules! enum_set_type_internal {
 /// While attributes and documentation can be attached to the enums, the variants may not
 /// contain data.
 ///
-/// `Copy` and `Clone` are automatically derived for the enums. Other traits must be manually
-/// derived or implemented.
-///
 /// # Examples
 ///
 /// ```rust
@@ -505,7 +481,6 @@ macro_rules! enum_set_type {
 /// # #[macro_use] extern crate enumset;
 /// # use enumset::*;
 /// # enum_set_type! {
-/// #     #[derive(Debug)]
 /// #     enum Enum { A, B, C }
 /// # }
 /// # fn main() {
@@ -529,7 +504,6 @@ mod test {
     use super::*;
 
     enum_set_type! {
-        #[derive(Debug)]
         enum Enum {
             A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
         }
@@ -538,9 +512,8 @@ mod test {
         }
     }
 
-    #[cfg(feature = "i128")]
+    #[cfg(feature = "nightly")]
     enum_set_type! {
-        #[derive(Debug)]
         enum LargeEnum {
             _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
             _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
@@ -570,7 +543,7 @@ mod test {
         A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
     }
 
-    #[cfg(feature = "i128")]
+    #[cfg(feature = "nightly")]
     test_variants! { LargeEnum large_enum_variant_range_test
         _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
         _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
@@ -665,6 +638,6 @@ mod test {
     }
 
     test_enum!(Enum, small_enum);
-    #[cfg(feature = "i128")]
+    #[cfg(feature = "nightly")]
     test_enum!(LargeEnum, large_enum);
 }
